@@ -1780,8 +1780,13 @@ int parse_mixed_barcode_data(char *data)
     char *urlp;
     // the assumption here is that the barcode is definitely not an EAN
 
+
+    // this link used for identifiers and ideas.
     // http://code.google.com/p/zxing/wiki/BarcodeContents
 
+
+
+
     printf("Mixed barcode: %s\n", data);
 
     strcpy(appdata->EAN_text, data);
@@ -1790,7 +1795,12 @@ int parse_mixed_barcode_data(char *data)
     strcpy(appdata->price_text, "");
     update_scan_window();
 
-    // check for a vCard, BIZCARD (like the MECARD message),
+
+
+
+
+
+    // check for a vCard
     urlp = strstr(data, "BEGIN:VCARD");
     if (urlp){
         // we assume there's nothing of interest on the end
@@ -1804,51 +1814,7 @@ int parse_mixed_barcode_data(char *data)
 
 
 
-    
-    // see if this contains an URL
-    
-    // check for the URL: header
-    urlp = strstr(data, "URL:");
-    if (urlp){
-        urlp = urlp+4; // skip the URL: business
-        
-        printf("URL!\n");
-        // we assume there's nothing of interest on the end
-        openURL(urlp); // open the url
-        return TRUE;
-    }
-    // check for the URLTO: header
-    urlp = strstr(data, "URLTO:");
-    if (urlp){
-        urlp = urlp+6; // skip the URLTO: business
-        
-        printf("URL!\n");
-        // we assume there's nothing of interest on the end
-        openURL(urlp); // open the url
-        return TRUE;
-    }
-    // check for a plain http://
-    urlp = strstr(data, "http://");
-    if (urlp){        
-        printf("URL!\n");
-        // we assume there's nothing of interest on the end
-        openURL(urlp); // open the url
-        return TRUE;
-    }
-    
-    // check for an email address...
-    
-    
-    // mms: MMSTO
-    
-    // sms: SMSTO: ??
-    
-    // geo:
-    
-    // youtube://
-    
-    // iCal
-    
+
     // if it's QR code, check the iMode stuff
     // http://www.nttdocomo.co.jp/english/service/imode/make/content/barcode/function/content/index.html
     
@@ -1866,11 +1832,124 @@ int parse_mixed_barcode_data(char *data)
         return TRUE;
     }else if (strncmp(data, "MEBKM:", 6)){
         printf("Bookmark registration message\n");
+        // we could try parsing this out and asking the user if they want to save it as a bookmark, or just open it in the browser
         return TRUE;
     }else if (strncmp(data,"LAPL:", 5)){
         printf("i-appli sync message\n");
         return TRUE;
     }
+
+
+
+    
+    // see if this contains an URL
+    // we should check for this last, as there may be an URL in e.g. a VCard
+    
+    // check for the URL: header
+    urlp = strstr(data, "URL:");
+    if (urlp){
+        urlp = urlp+4; // skip the URL: business
+        
+        printf("URL!\n");
+        // we assume there's nothing of interest on the end
+        openURL(urlp); // open the url
+        return TRUE;
+    }
+    
+    // check for the URLTO: header
+    urlp = strstr(data, "URLTO:");
+    if (urlp){
+        urlp = urlp+6; // skip the URLTO: business
+        
+        printf("URL!\n");
+        // we assume there's nothing of interest on the end
+        openURL(urlp); // open the url
+        return TRUE;
+    }
+    
+    // check for a plain http://
+    urlp = strstr(data, "http://");
+    if (urlp){        
+        printf("URL!\n");
+        // we assume there's nothing of interest on the end
+        openURL(urlp); // open the url
+        return TRUE;
+    }
+    
+    
+    
+    // check for an email address...
+    if (strncmp(data, "mailto:", 7)){        
+        printf("Email address\n");
+        return TRUE;
+    }
+
+
+    // check for a telephone number...
+    if (strncmp(data, "tel:", 4)){      
+        printf("Email address\n");
+        return TRUE;
+    }
+    
+
+
+    // check for a BIZCARD
+    if (strncmp(data, "BIZCARD:", 8)){      
+        printf("BIZCARD\n");
+        return TRUE;
+    }
+
+
+    // check for an SMS
+    if (strncmp(data, "sms:", 4)){      
+        printf("SMS\n");
+        return TRUE;
+    }
+    if (strncmp(data, "smsto:", 6)){
+        printf("SMS\n");
+        return TRUE;
+    }
+
+
+    // check for an MMS
+    if (strncmp(data, "mms:", 4)){      
+        printf("MMS\n");
+        return TRUE;
+    }
+    if (strncmp(data, "mmsto:", 6)){
+        printf("MMS\n");
+        return TRUE;
+    }
+    
+
+    // check for geographic information
+    if (strncmp(data, "geo:", 4)){      
+        printf("Geographical information\n");
+        return TRUE;
+    }
+
+
+
+    // Android specific - link to app in Market
+    if (strncmp(data, "market:", 7)){      
+        printf("Android Market link\n");
+        return TRUE;
+    }
+
+
+
+    // iPhone rumoured - link to YouTube
+    if (strncmp(data, "youtube:", 8)){      
+        printf("Link to YouTube\n");
+        return TRUE;
+    }
+
+
+    // youtube://
+    
+    // iCal
+    
+
     
     return TRUE;
 
@@ -2097,17 +2176,17 @@ int handle_EAN8(char *data)
 // I've been naughty here and have hardcoded the method names etc. but
 // these are all defined in /usr/include/tablet-browser-interface.h
 
+
+
+#ifdef N900
+
 void openURL(char *open)
 {
     GtkWidget *dialog;
     gchar *uri = g_strdup(open);
     
-    
-    
     printf("Opening URL in browser: %s\n", uri);
 
-    
-
     dialog = gtk_message_dialog_new(GTK_WINDOW(appdata->window),
             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
             GTK_MESSAGE_QUESTION,
@@ -2115,12 +2194,12 @@ void openURL(char *open)
             "Open URL: %s", uri);
             
     g_signal_connect(dialog, "response", G_CALLBACK (openURL_response_cb), uri);
-        
+    gtk_window_set_transient_for  (GTK_WINDOW (dialog), GTK_WINDOW (appdata->window));
     gtk_widget_show_all (dialog);
     
 }
 
-#ifdef N900
+
 void openURL_response_cb (GtkDialog *dialog, gint response, gpointer user_data)
 {
     char *uri = (char*)user_data;
@@ -2161,10 +2240,33 @@ void openURL_response_cb (GtkDialog *dialog, gint response, gpointer user_data)
 }
 
 #else
+
+void openURL(char *open)
+{
+    GtkWidget *dialog;
+    gchar *uri = g_strdup(open);
+    
+    
+    
+    printf("Opening URL in browser: %s\n", uri);
+
+    
+
+    dialog = gtk_message_dialog_new(GTK_WINDOW(appdata->window),
+            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+            GTK_MESSAGE_QUESTION,
+            GTK_BUTTONS_YES_NO,
+            "Open URL: %s", uri);
+            
+    g_signal_connect(dialog, "response", G_CALLBACK (openURL_response_cb), uri);
+        
+    gtk_widget_show_all (dialog);
+    
+}
+
 void openURL_response_cb (GtkDialog *dialog, gint response, gpointer user_data)
 {
     char *uri = (char*)user_data;
-    osso_context_t* osso_context;
     osso_rpc_t* osso_retval = { 0 };
 
     gtk_widget_destroy(dialog);
@@ -2173,20 +2275,19 @@ void openURL_response_cb (GtkDialog *dialog, gint response, gpointer user_data)
     switch (response) {
         case GTK_RESPONSE_YES:
             /*Create the osso context for the browser application.*/
-            osso_context = osso_initialize("org.maemo.mbarcode", "0.0.1", FALSE, NULL);
             /*Call the given function.*/
             
             printf("Open URL: %s, length=%d\n", uri, strlen(uri));
             
-            osso_rpc_run(osso_context, "com.nokia.osso_browser","/com/nokia/osso_browser",
+            osso_rpc_run(appdata->osso, "com.nokia.osso_browser","/com/nokia/osso_browser",
             "com.nokia.osso_browser","open_new_window", NULL, DBUS_TYPE_STRING, uri, DBUS_TYPE_INVALID);
 
 /*
-            osso_rpc_run(osso_context, "com.nokia.osso_browser","/com/nokia/osso_browser/request",
+            osso_rpc_run(appdata->osso, "com.nokia.osso_browser","/com/nokia/osso_browser/request",
             "com.nokia.osso_browser","load_url", NULL, DBUS_TYPE_STRING, uri, DBUS_TYPE_INVALID);
 */
             /*Free all memory allocated*/
-            osso_deinitialize(osso_context);
+
             break;
         default:
             break; // dialog was cancelled, so do nothing
